----------
-- Locals
----------

local tooltip = CrowBarTooltip
local busy = nil
local defaults = {
	profile = {}
}
local db
local opening = GetSpellInfo(6247)

local ITEM_OPEN_CLAM = 'Use: Open the clam!'

if GetLocale() == "deDE" then
	ITEM_OPEN_CLAM = 'Benutzen: \195\150ffnet die Muschel!'
end

-----------
-- Addon Declaration
-----------

CrowBar = LibStub("AceAddon-3.0"):NewAddon("CrowBar", "AceEvent-3.0", "AceTimer-3.0", "AceBucket-3.0")
local addon = CrowBar

-----------
-- Addon Initialization
-----------

function addon:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("CrowBarDB", defaults, "Default")
	db = self.db
end

function addon:OnEnable()
	self:SetupButton()
	self:RegisterBucketEvent("BAG_UPDATE", 0.5, "ScanBags")
	-- listen to locksmith scan bag event to update ourself when a box is unlocked
	self:RegisterMessage("LockSmith_ScanBags", "ScanBags")
	self:RegisterEvent("PLAYER_REGEN_ENABLED", "ScanBags")
	self:RegisterEvent("UNIT_SPELLCAST_SENT", "OpenStart")
	self:RegisterEvent("UNIT_SPELLCAST_FAILED", "OpenStop")
	self:RegisterEvent("UNIT_SPELLCAST_STOP", "OpenStop")
	self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED", "OpenStop")
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED", "OpenStop")
	
	self:ScanBags()
end

function addon:OnDisable()
	self:HideButton()
end

------------
-- The gory innards
------------

local function FindOpenable()
	for i=1,tooltip:NumLines() do
		if tooltip.L[i] == ITEM_OPENABLE or tooltip.L[i] == ITEM_OPEN_CLAM then
			return true
		end
	end
	return false
end

function addon:ScanBags()
	if InCombatLockdown() then return end
	local itemid

	local boxbag, boxslot = nil,nil

	for bag = 0, 4, 1 do
		for slot = 1, GetContainerNumSlots(bag), 1 do
			if not boxbag then
				tooltip:SetBagItem( bag, slot )
				if FindOpenable() then
					boxbag = bag
					boxslot = slot
					break
				end
			end
		end
		if boxbag then
			break
		end
	end
	if boxbag then
		self:SetButton(boxbag, boxslot)
	else
		self:HideButton()
	end
end

function addon:OpenStart(event, unit, spell, rank, target )
	if unit == "player" and spell == opening then
		busy = true
	end
end

function addon:OpenStop(event, unit)
	if unit == "player" and busy then
		busy = nil
		self:ScheduleTimer(self.ScanBags, 1, self ) -- update after a second.
	end
end

function addon:SetButton(bag, slot)
	if InCombatLockdown() then 
		self:HideButton()
		return
	end
	-- secure attributes!
	local button = self.button
	button:SetAttribute( "type1", "macro" )
	button:SetAttribute( "macrotext1", "/use "..bag.." "..slot)
	button:Show()
end

function addon:HideButton()
	if not self.button then return end
	self.button:Hide()
end

function addon:SetupButton()
	if self.button then return end

	self.button = CreateFrame("Button", "CrowBarButton", UIParent, "SecureActionButtonTemplate, ActionButtonTemplate")


	local button = self.button

	button:Hide()
	button:SetWidth( 32 )
	button:SetHeight( 32 )
	button:ClearAllPoints()
	button:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	button:EnableMouse(true)
	button:RegisterForDrag("LeftButton")
	button:RegisterForClicks("AnyUp")
	button:SetMovable(true)
	button:SetScript("OnDragStart", function() if not InCombatLockdown() and IsAltKeyDown() then button:StartMoving() end end)
	button:SetScript("OnDragStop", function() if not InCombatLockdown() then button:StopMovingOrSizing() addon:SavePosition() end end)

	button.icon = CrowBarButtonIcon
	button.icon:SetTexture( "Interface\\Icons\\INV_Box_02" )

	self:RestorePosition()
end

function addon:SavePosition()
	local f = self.button
	local s = f:GetEffectiveScale()

	self.db.profile.posx = f:GetLeft() * s
	self.db.profile.posy = f:GetTop() * s
end

function addon:RestorePosition()
	local x = self.db.profile.posx
	local y = self.db.profile.posy

	if not x or not y then return end

	local f = self.button
	local s = f:GetEffectiveScale()

	f:ClearAllPoints()
	f:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x / s, y / s)
end
